unit Globals;
interface
uses DBTables,Classes;

type
   {create a subclass to hold global info}
   {TGlobalComponent is the ancestor of TApplication}
   TAppGlobals = Class(TForm){(TComponent)}
     private
        {fields, methods not visible to others}
        ProfileTable : TTable;
        FName : String;
        FMealCost: Double;
        FVersion : String;
        {CCopyright  }
        FOwner : String;
        FComment: String;
        {Read_only}
        FAlreadyDone : Boolean; {String;}
        FDSDATEI : Integer;
        FDayIS : String;
        FAppComment: String;
        FMonthIS : String;
        FMonthDays: Shortint;
        FWeekBegins :TDateTime;
        FWeekEnds: TDateTime;
        FConfigTable : String;
        FConfigDb : String;
        function GetConfigTable: String;
        procedure SetConfigTable(const TableName: String);
        function GetConfigDb: String;
        procedure SetConfigDb(const Db: String);
        function GetVersion: String;
        function GetName: String;
        function GetOwner: String;
        function  GetDSDATEI:Integer;
        Procedure SetDSDATEI(const SInDSDATEI:Integer);
        function GetDayIS: String;
        Procedure SetDayIS(const SInDayIS: String);
        function GetAppComment: String;
        Procedure SetAppComment(const SINAppComment: String);
        function GetMonthIS: String;
        Procedure SetMonthIS(const SInMonthIS: String);
        function GetMonthDays : Shortint;
        Procedure SetMonthDays(const SInMonthDays: Shortint);
        function GetWeekBegins :TDateTime;
        Procedure SetWeekBegins(const SinWeekBegins :TDateTime);
        function GetWeekEnds: TDateTime;
        Procedure SetWeekEnds(const SinWeekEnds :TDateTime);
        function GetMealCost: Double;
        Procedure SetMealCost(const SinMealcost: Double);
        function GetBuild: String;
        function GetCopyright: String;
        function GetAlreadyDone:Boolean; {String;}
        Procedure SetAlreadyDone(const SAlreadyDone:Boolean);
     protected
     public
        Constructor Create(Sender: TObject){(Owner:TComponent)};
        Procedure ReadCfg;
        Procedure WriteCfg;

     published
        property Version: String read GetVersion;
        property Name: String read GetName;
        property Mealcost: Double read GetMealCost write SetMealCost;
        property Owner: String read GetOwner;
        property Build: String read GetBuild;
        property Copyright: String read GetCopyright;
        property DSDATEI: Integer read GetDSDATEI write SetDSDATEI;
        property AlreadyDone: Boolean{String} read GetAlreadyDone
                                               write SetAlreadyDone;
        property DayIS: String read GetDayIS
                               write SetDayIS;
        property AppComment: String read GetAppComment write SetAppComment;

        property MonthIS: String read GetMonthIS write SetMonthIS;
        property MonthDays: Shortint read GetMonthDays write SetMonthDays;
        property WeekBegins: TDateTime read GetWeekBegins write SetWeekBegins;
        property WeekEnds: TDateTime read GetWeekEnds write SetWeekEnds;
        property ConfigTable : String read GetConfigTable
                                      write SetConfigTable;
        property ConfigDb : String read GetConfigDb
                                   write SetConfigDb;
     end;

Var
  {global variable}
  AppGlobals : TAppGlobals;
  ThisWeekBegins, ThisWeekEnds : TDateTime;
{  DSDATE:Integer;}
implementation
const
   {constants used by some of the properties}
   CCopyright = 'Copyright 1998, Ivan Lee Herring';
   CBuild = '1.50b       ilh2o';

constructor TAppGlobals.Create(Sender: TObject){(AOwner:TComponent)};
begin
   {execute the ancestor constructor}
   inherited Create(nil){(Aowner)};
   {read the config table settings}
   ReadCfg;
end;


procedure TAppGlobals.ReadCfg;
{var AlStr: String;}
begin
   {assign the config table name and database}
   FConfigTable:='F:\mow\data\Syscfg.DB';
   FConfigDb:='';
   {create the TTable object using self as the owner
    with self as the owner, when the owner is destroyed
    ProfileTable will be destroyed}
   ProfileTable := TTable.Create(self);
   {set the tablename property}
   ProfileTable.TableName := FConfigTable;
   {if a database name is given, use it}
   if FConfigDb <> '' then
      ProfileTable.DatabaseName := FConfigDb;
   {Open the profile table}
   ProfileTable.Open;
   {store the field values into the object fields}
   FMealcost := ProfileTable.FieldByName('Meal_Cost').AsFloat;
   FOwner := ProfileTable.FieldByName('Owner_name').AsString;
   FDSDATEI:= ProfileTable.FieldByName('DSDATEI').AsInteger;
{DSDATE:=FDSDATEI;}
   FDayIS := ProfileTable.FieldByName('DayIS').AsString;
   FAppComment := ProfileTable.FieldByName('App_Comment').AsString;
   FMonthIS := ProfileTable.FieldByName('MonthIS').AsString;
   FMonthDays := ProfileTable.FieldByName('MonthDays').AsInteger;
   FWeekBegins := ProfileTable.FieldByName('WeekBegins').AsDateTime;
   FWeekEnds := ProfileTable.FieldByName('WeekEnds').AsDateTime;
ThisWeekBegins :=FWeekBegins;
ThisWeekEnds := FWeekEnds;
   FAlreadyDone := ProfileTable.FieldByName('AlreadyDone_Log').AsBoolean; {AsString;}
   FVersion := ProfileTable.FieldByName('Version_number').AsString;
   FName := ProfileTable.FieldByName('Application_name').AsString;
   {close the profile table}
   ProfileTable.close;
end;

procedure TAppGlobals.WriteCfg; {See ReadCfg for notes}
begin
   FConfigTable:='F:\mow\data\Syscfg.DB';{   'SYSCFG.DB';}
   FConfigDb:='';
   ProfileTable := TTable.Create(self);
   ProfileTable.TableName := FConfigTable;
   if FConfigDb <> '' then
      ProfileTable.DatabaseName := FConfigDb;
   ProfileTable.Open;
   ProfileTable.Edit;
{   FDSDATEI:=DSDATE;}
   ProfileTable['DSDATEI'] :=FDSDATEI;
   ProfileTable['Meal_Cost'] := FMealcost;
   ProfileTable['DayIS'] := FDayIS;
   ProfileTable['App_Comment'] := FAppComment;
   ProfileTable['MonthIS'] := FMonthIS;
    ProfileTable['MonthDays']:=FMonthDays;
   {FWeekBegins:= ThisWeekBegins; }
   ProfileTable['WeekBegins'] := FWeekBegins;
{FWeekEnds:=ThisWeekEnds; }
   ProfileTable['WeekEnds'] := FWeekEnds;
   ProfileTable['AlreadyDone_Log']  := FAlreadyDone;
   {close the profile table}
   ProfileTable.Post;
   ProfileTable.close;
end;


function TAppGlobals.GetVersion;
begin
   GetVersion:=FVersion;
end;

function TAppGlobals.GetBuild;
begin
   GetBuild:=CBuild;
end;

function TAppGlobals.GetName;
begin
   GetName:=FName;
end;
       function  TAppGlobals.GetDSDATEI:Integer;
       Begin
       GetDSDATEI:=FDSDATEI;
       End;
        Procedure TAppGlobals.SetDSDATEI(const SinDSDATEI:Integer);
       Begin
       FDSDATEI:= SinDSDATEI;
       End;

        function TAppGlobals.GetMealCost: Double;
        Begin
          GetMealCost:= FMealCost;
        End;
        Procedure TAppGlobals.SetMealCost(const SinMealcost: Double);
        Begin
          FMealCost := SinMealcost;
        End;
        function TAppGlobals.GetMonthDays : Shortint;
        Begin
          GetMonthDays:= FMonthDays;
        End;
        Procedure TAppGlobals.SetMonthDays(const SInMonthDays: Shortint);
        Begin
           FMonthDays:= SInMonthDays;
        End;
        function TAppGlobals.GetAppComment;
        begin
          GetAppComment:=FAppComment;
        end;
        Procedure TAppGlobals.SetAppComment(const SINAppComment: String);
   begin
  FAppComment := SINAppComment;
end;

function TAppGlobals.GetOwner;
begin
  GetOwner:=FOwner;
end;
function TAppGlobals.GetDayIS;
begin
  GetDayIS :=FDayIS;
end;
Procedure TAppGlobals.SetDayIS(const SInDayIS: String);
begin
  FDayIS := SInDayIS;
end;
        function TAppGlobals.GetWeekBegins :TDateTime;
        Begin
        GetWeekBegins := FWeekBegins;
        End;
        Procedure TAppGlobals.SetWeekBegins(const SinWeekBegins :TDateTime);
        Begin
         FWeekBegins:= SinWeekBegins;
        End;

        function TAppGlobals.GetWeekEnds: TDateTime;
        Begin
        GetWeekEnds:=FWeekEnds;
        End;

        Procedure TAppGlobals.SetWeekEnds(const SinWeekEnds :TDateTime);
        Begin
        FWeekEnds := SinWeekEnds;
        End;


function TAppGlobals.GetMonthIS;
begin
  GetMonthIS :=FMonthIS;
end;
Procedure TAppGlobals.SetMonthIS(const SInMonthIS: String);
begin
  FMonthIS := SInMonthIS;
end;

function TAppGlobals.GetAlreadyDone;
begin
  GetAlreadyDone:=FAlreadyDone;
end;
Procedure TAppGlobals.SetAlreadyDone(const SAlreadyDone: Boolean);
begin
  FAlreadyDone:=SAlreadyDone;
end;

function TAppGlobals.GetCopyright;
begin
   GetCopyright:=CCopyright;
end;

function TAppGlobals.GetConfigTable;
begin
   GetConfigTable:=FConfigTable;
end;

procedure TAppGlobals.SetConfigTable(const TableName: String);
begin
   FConfigTable := TableName;
end;

function TAppGlobals.GetConfigDb;
begin
   GetConfigDb:=FConfigDb;
end;

procedure TAppGlobals.SetConfigDb(const Db: String);
begin
   FConfigDb := Db;
end;


initialization
   AppGlobals:= TAppGlobals.Create(Sender: TObject){(Owner)};
end.
