unit PmUtil;

interface
Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,Forms,Dialogs,
  StdCtrls, ComCtrls, IniFiles;

Type

Inrec = Record // IniFile record
 PMmapCfg : String[80];
 Contour : String[80];
 Item0Lon : String[10];
 Item0Lat : String[10];
 Item0Str : String[30];
 Item1Lon : String[10];
 Item1Lat : String[10];
 Item1Str : String[30];

End;  // IniFile

Zrec = Record   // FindZipCode
 ZipCode : String[10];
 City : String[30];
 State : String[2];
 AreaCode : String[3];
 Lon : Double;
 Lat : Double;
End;  // Zrec

AcRec = Record  // FindAcEx
 AreaCode : String[3];
 Exchange : String[3];
 City : String[30];
 Lon : Double;
 Lat : Double;
End;  // AcRec

Crec = Record   // FindCity
 City : String[30];
 State : String[2];
 Lon : Double;
 Lat : Double;
End;  // Crec

SRec = Record   // FindStr
 Street : String[30];
 XStreet : String[30];
 Add : Integer;
 LonCtr : Double;
 LatCtr : Double;
 mRadius : Double;
 StStr : String[30];
 Zoom : Double;
 City : String[30];
 ZipCode : String[10];
End;  // Srec

IRec = Record   // Item Record
 Id : Integer;
 Lon : Double;
 Lat : Double;
 Comment : String[30];
End;  // Irec

Brec = Record  // Bitmap or Pcc record
 Fname : String[40];
 NwLon : Double;
 NwLat : Double;
 SeLon : Double;
 SeLat : Double;
 Shadow : Integer;
 Style : Integer;
End;  // Brec

LLRect =  Record      // Latitude longitude rectangle
 TopLeftLon : Double; // Win Api calls using bounding rectangle
 TopLeftLat : Double; // use the top Left and bottom right pixel coordinates.
 TopLeftX : Integer;
 TopLeftY : Integer;
 BotRightLon : Double;
 BotRightLat : Double;
 BotRightX : Integer;
 BotRightY : Integer;
End;  // LLRect

Function GetRadiusRect(CtrLon,CtrLat,smiles : Double;
                       User : Integer; Var Drw : LLRect) : Double;
Function CommaDelStr(var delstr : String) : String;
Function CommaDelInt(Var delstr : String) : Integer;
Function CommaDelFlt(Var delstr : String) : Double;
Procedure IniRead;
Procedure IniWrite;
Procedure GetMapConfig(CfgName : String);
Function TabDelim(Var TabStr : String) : String;
Function TabDelInt(Var delstr : String) : Integer;
Function TabDelFlt(Var delstr : String) : Double;

Var
ItemRec : Array[0..9] of Irec;
IniRec : InRec;  // Inifile record
ItemCntr : Integer;
FCityRec : Crec;  // FindCity record
FAcRec : AcRec;   // FindAreaCode Record
FZipRec : ZRec;   // FindZipCode Record
FStrRec : SRec;   // FindStr record
PccRec : BRec;
BmpRec : Brec;
FindList : TStringList;
ARatio : Double = 1.0;
FirstTime : Boolean = True;
implementation

Uses Pm;


Function GetRadiusRect(CtrLon,CtrLat,smiles : Double;
                       User : Integer; Var Drw : LLRect) : Double;
 // Determine a bounding rectangle for drawing operations
 // using center Lon,Lat, radius in miles,
 // User = 0 use lon, 1 use lat, ,2 Use both, or 3 use avg.
 Var Td,Td1,Td2,Tm : Double;
Begin
 with Main.OlePmMap1 do
 Begin
  if smiles = 0 then
  Begin
   GetRadiusRect := 0.0;  // Error must supply radius
   Exit;
  End;

  Tm := smiles * 1000; // Convert to thousandths
  Td := Distance(CtrLon,CtrLat,CtrLon+1.0,CtrLat); //  1 degree of lon at this lat.
  Td1 := (Tm/Td)/2;

  Td2 := Distance(CtrLon,CtrLat,CtrLon,CtrLat+1.0);// Distance for 1 deg of Lat
  Td2 := (Tm/Td2)/2;

  With Drw do
  Case User of
  0 : Begin  // Use calculated lon distance only
       TopLeftLon :=  CtrLon + Td1;
       TopLeftLat := CtrLat  + Td1;
       BotRightLon := CtrLon - Td1;
       BotRightLat := CtrLat - Td1;
      End;
  1:  Begin  // Use calculated Lat distance only
       TopLeftLon :=  CtrLon + Td2;
       TopLeftLat := CtrLat  + Td2;
       BotRightLon := CtrLon - Td2;  
       BotRightLat := CtrLat - Td2;
      End;
  2:  Begin  // Use both calculated distances.  Most accurate
       TopLeftLon :=  CtrLon + Td1;
       TopLeftLat := CtrLat  + Td2;
       BotRightLon := CtrLon - Td1;
       BotRightLat := CtrLat - Td2;
      End;
  3: Begin // Use avg of calculated distances
      Td1 := (Td1 + Td2) / 2;
      TopLeftLon :=  CtrLon + Td1;
      TopLeftLat := CtrLat  + Td1;
      BotRightLon := CtrLon - Td1;
      BotRightLat := CtrLat - Td1;
     End;
  End;  // Case

  if td <> 0 then
    with Drw do
    Begin
     TopLeftX := Lon2Int(TopLeftLon);
     TopLeftY := Lat2Int(TopLeftLat);
     BotRightX := Lon2Int(BotRightLon);
     BotRightY := Lat2Int(BotRightLat);
     GetRadiusRect := td; // Non-Zero = ok
    End
   else
    GetRadiusRect := 0.0;
 End;
End;  // GetRadiusRect

Function CommaDelStr(var delstr : String) : String;
// Return substr up to first comma in delstr &
// delete those char including comma
Begin
 delstr := Trim(delstr);
 If Pos(',',DelStr) > 0 then
 Begin
  Result := Copy(DelStr,1,Pos(',',DelStr)-1);
  Delete(DelStr,1,Pos(',',DelStr));
 End
  else
 Begin
  Result := delstr;
  DelStr := '';
 End;
End;  // CommaDelStr

Function CommaDelInt(Var delstr : String) : Integer;
// Return Int or 0 contained in delstr up to first comma
// delete those char including comma
Begin
 delstr := Trim(delstr);
 If Pos(',',DelStr) > 0 then
 Begin
  Result := StrToIntDef(Copy(DelStr,1,Pos(',',DelStr)-1),0);
  Delete(DelStr,1,Pos(',',DelStr));
 End
 Else
 Begin
  Result := StrToIntDef(DelStr,0);
  DelStr := '';
 End;
End;  // CommaDelInt

Function CommaDelFlt(Var delstr : String) : Double;
// Return Float or 0 contained in delstr up to first comma
// delete those char including comma
Var cntr : Integer;
tstr : String;
Begin
 Result := 0.0; // Default return
 Delstr := Trim(DelStr);
 if Pos(',',Delstr) > 0 then
 Begin
  tstr := Copy(DelStr,1,Pos(',',DelStr)-1);
  Delete(DelStr,1,Pos(',',DelStr));
 End
 Else
 Begin
  tstr := DelStr;
  DelStr := '';
 End;
 If Length(tstr) = 0 then exit;

 For cntr := 1 to Length(tstr) do
  if Pos(tstr[cntr],'-0123456789.')= 0 then exit; // must be digit dec point etc

  Result := StrToFloat(tstr);
End;  // CommaDelFlt


 Procedure IniRead;
 { Reads App.ini in application directory }
  var
   dstr,fstr,cstr : string;
   inifile : TiniFile;
  Begin
  Try
   Try
    dstr := ExtractFilePath(application.exename);
    fstr := ExtractFileName(application.exename);
    fstr := Copy(fstr,1,Pos('.',fstr)) + 'ini';
    inifile := TiniFile.create(dstr + fstr);
    cstr := dstr + 'Contour\Pmap.Pcc';
   With IniRec,IniFile do
   Begin
    PMmapCfg := ReadString('Init','PMmapCfg','');
    Contour := ReadString('Init','Contour',cstr);
    Item0Lon := ReadString('Init','Item0Lon','-88.0546');
    Item0Lat := ReadString('Init','Item0Lat','41.5725');
    Item0Str := ReadString('Init','Item0Str','Test 0');
    Item1Lon := ReadString('Init','Item1Lon','-88.051');
    Item1Lat := ReadString('Init','Item1Lat','41.5679');
    Item1Str := ReadString('Init','Item1Str','Test 1');
    Main.OlePmMap1.SaveView :=  ReadBool('Init','SaveView',True);
    Main.LoadLastView1.Checked :=  Main.OlePmMap1.SaveView;

    ItemRec[0].Lon := StrToFloat(Item0Lon);
    ItemRec[0].Lat := StrToFloat(Item0Lat);
    ItemRec[0].Comment := Item0Str;
    ItemRec[1].Lon := StrToFloat(Item1Lon);
    ItemRec[1].Lat := StrToFloat(Item1Lat);
    ItemRec[1].Comment := Item1Str;
    
    BmpRec.Fname := ReadString('Bmp','Fname','');
    BmpRec.NwLon := StrToFloat(ReadString('Bmp','NwLon','0.0'));
    BmpRec.NwLat := StrToFloat(ReadString('Bmp','NwLat','0.0'));
    BmpRec.SeLon := StrToFloat(ReadString('Bmp','SeLon','0.0'));
    BmpRec.SeLat := StrToFloat(ReadString('Bmp','SeLat','0.0'));
    BmpRec.Shadow := ReadInteger('Bmp','Shadow',0);
    BmpRec.Style := ReadInteger('Bmp','Style',0);

    PccRec.Fname := ReadString('Pcc','Fname','');
    PccRec.NwLon := StrToFloat(ReadString('Pcc','NwLon','0.0'));
    PccRec.NwLat := StrToFloat(ReadString('Pcc','NwLat','0.0'));
    PccRec.SeLon := StrToFloat(ReadString('Pcc','SeLon','0.0'));
    PccRec.SeLat := StrToFloat(ReadString('Pcc','SeLat','0.0'));
    PccRec.Shadow := ReadInteger('Pcc','Shadow',0);
    PccRec.Style := ReadInteger('Pcc','Style',0);

   End;
  except
    messagedlg('IniFile Read Error ' + Chr(10) ,mterror,[mbok],0);
   end;
 Finally
  IniFile.Free;
 End;
end; { IniRead }

 Procedure IniWrite;
  { Writes all nec. variables to ini file }
  var
    inifile : tinifile;
    fstr,dstr : string;
  Begin
   try
    Try
     dstr := ExtractFilePath(application.exename);
     fstr := ExtractFileName(application.exename);
     fstr := Copy(fstr,1,Pos('.',fstr)) +'ini';
     inifile := TiniFile.create(dstr + fstr);

     With IniFile,IniRec do
     Begin
      Item0Lon := FloatToStr(ItemRec[0].Lon);
      Item0Lat := FloatToStr(ItemRec[0].Lat);
      Item0Str := ItemRec[0].Comment;
      Item1Lon := FloatToStr(ItemRec[1].Lon);
      Item1Lat := FloatToStr(ItemRec[1].Lat);
      Item1Str := ItemRec[1].Comment;

      WriteString('Init','PmMapCfg',PmMapCfg);
      WriteString('Init','Contour',Contour);
      WriteString('Init','Item0Lon',Item0Lon);
      WriteString('Init','Item0Lat',Item0Lat);
      WriteString('Init','Item0Str',Item0Str);
      WriteString('Init','Item1Lon',Item1Lon);
      WriteString('Init','Item1Lat',Item1Lat);
      WriteString('Init','Item1Str',Item1Str);
      WriteBool('Init','SaveView',Main.OlePmMap1.SaveView);

      WriteString('Bmp','Fname',BmpRec.Fname);
      WriteString('Bmp','NwLon',FloatToStr(BmpRec.NwLon));
      WriteString('Bmp','NwLat',FloatToStr(BmpRec.NwLat));
      WriteString('Bmp','SeLon',FloatToStr(BmpRec.SeLon));
      WriteString('Bmp','NwLat',FloatToStr(BmpRec.SeLat));
      WriteInteger('Bmp','Shadow',BmpRec.Shadow);
      WriteInteger('Bmp','Style',BmpRec.Style);

      WriteString('Pcc','Fname',PccRec.Fname);
      WriteString('Pcc','NwLon',FloatToStr(PccRec.NwLon));
      WriteString('Pcc','NwLat',FloatToStr(PccRec.NwLat));
      WriteString('Pcc','SeLon',FloatToStr(PccRec.SeLon));
      WriteString('Pcc','NwLat',FloatToStr(PccRec.SeLat));
      WriteInteger('Pcc','Shadow',PccRec.Shadow);
      WriteInteger('Pcc','Style',PccRec.Style);


     End;

   except
     messagedlg('IniFile Write Error ',mterror,[mbok],0);
   end;
   Finally
     inifile.free;
   End;
  end; { IniWrite }


Procedure GetMapConfig(CfgName : String);
 { Reads Pmap.Cfg and initializes OCX properties.}
  var
    ConFigfile : TiniFile;
    SatStr,Cstr : String;
    SatFile : TextFile;
    Err : Word;
  Begin
  Try
   Try

    ConFigfile := TiniFile.create(CfgName);
    IniRec.PmMapCfg := CfgName;
   With Main.OlePmMap1,ConFigfile do
   Begin
    Path_States := Configfile.ReadString('Directories','CdState','');
    Path_Data1 := ReadString('Directories','CdData1','');
    Path_Data2 := ReadString('Directories','CdData2','');
    Path_Data3 := ReadString('Directories','CdData3','');
    IniRec.Contour := ExtractFilePath(ReadString('Directories','CdCont',IniRec.Contour));
    Load_Hydro := ReadBool('Overlays','LHydro',True);
    Load_Highway := ReadBool('Overlays','LhNet',True);
    Load_County := ReadBool('Overlay','Lcounty',True);
    Load_State := ReadBool('Overlays','Lstate',True);
    Load_City := ReadBool('OverLays','LCity',True);
    Load_Air := ReadBool('Overlays','LAir',True);
    Load_Landmark := ReadBool('Overlays','LLand',true);
    Enabled := True;
   End;

   Cstr := ExtractFilePath(IniRec.Contour) + 'PMap.Sat';
   If FileExists(Cstr) Then
    With PccRec do
    Begin
     AssignFile(SatFile,cstr);
    {$I-}
    Reset(SatFile);     // Beginning of file
    {$I+}
    err := IORESULT;
    ReadLn(SatFile,SatStr);
    NwLon := CommaDelFlt(SatStr);
    NwLat := CommaDelFlt(SatStr);
    SeLon := CommaDelFlt(SatStr);
    SeLat := CommaDelFlt(SatStr);
    if (err <> 0) then   // Error opening file
    Begin
     MessageDlg('Error opening '+Cstr,MtError,[MbOk],0);
      CloseFile(SatFile);
     exit;
    End;   // Err

     CloseFile(SatFile);
    End;


  except
    messagedlg('Config Read Error ' + Chr(10) ,mterror,[mbok],0);
  end;
 Finally
  ConfigFile.Free;

 End;
 End;  //  GetMapConfig.

 Function TabDelim(Var TabStr : String) : String;
 { Parse a tab delimitted string.  Returns sub string up to }
 { but not including the first tab or Cr.  Remove substr from tabstr }
 Begin
  If Pos(#9,TabStr) > 0Then
   Begin
    result := Copy(TabStr,1,Pos(#9,TabStr)-1);              // Get char up to tab
    TabStr := Copy(TabStr,Pos(#9,TabStr)+1,Length(TabStr)); // Get char after tab
   End
  else Result := TabStr;
 End; // TabDelim

 Function TabDelInt(Var delstr : String) : Integer;
// Return Int or 0 contained in delstr up to first tab
// delete those char including tab
Begin
 delstr := Trim(delstr);
 If Pos(#9,DelStr) > 0 then
 Begin
  Result := StrToIntDef(Copy(DelStr,1,Pos(#9,DelStr)-1),0);
  Delete(DelStr,1,Pos(#9,DelStr));
 End
 Else
 Begin
  Result := StrToIntDef(DelStr,0);
  DelStr := '';
 End;
End;  // CommaDelInt

 Function TabDelFlt(Var delstr : String) : Double;
// Return Float or 0 contained in delstr up to first Tab
// delete those char including Tab
Var cntr : Integer;
tstr : String;
Begin
 Result := 0.0; // Default return
 Delstr := Trim(DelStr);
 if Pos(#9,Delstr) > 0 then
 Begin
  tstr := Copy(DelStr,1,Pos(#9,DelStr)-1);
  Delete(DelStr,1,Pos(#9,DelStr));
 End
 Else
 Begin
  tstr := DelStr;
  DelStr := '';
 End;
 If Length(tstr) = 0 then exit;

 For cntr := 1 to Length(tstr) do
  if Pos(tstr[cntr],'-0123456789.')= 0 then exit; // must be digit dec point etc

  Result := StrToFloat(tstr);
End;  // TabDelFlt




end.
